import * as crypto from 'crypto';

export function decrypt(string = '', privateKeys = []) {
        // Converts the string to a buffer.
        let decryptedString = Buffer.from(string, 'base64');

        // Decrypts the string using all of the decryption keys.
        for (const privateKey of privateKeys) {
                decryptedString = crypto.privateDecrypt({
                        key: privateKey,
                        padding: crypto.constants.RSA_PKCS1_PADDING
                }, decryptedString);
        };

        // Returns the decrypted string.
        return decryptedString.toString();
};

export function encrypt(string = '', publicKeys = []) {
        // Converts the string to a buffer.
        let encryptedString = Buffer.from(string);

        // Encrypts the string using all of the encryption keys.
        for (const publicKey of publicKeys) {
                encryptedString = crypto.publicEncrypt({
                        key: publicKey,
                        padding: crypto.constants.RSA_PKCS1_PADDING
                }, encryptedString);
        };

        // Returns the encrypted string.
        return encryptedString.toString('base64');
};

export function pair(modulus = 2048) {
        return crypto.generateKeyPairSync('rsa', {
                modulusLength: modulus
        });
};

// Creates an object containing both an array of public keys and an array of private keys.
export function ring(modulus = 2048, quantity = 1) {
        // Defines the current modulus for later increments.
        let currentModulus = modulus;

        // Initializes the key arrays.
        const privateKeys = [];
        const publicKeys = [];

        // Creates the defined quantity of keys, progressively getting bigger based on padding.
        for (let i = 0; i < quantity; i += 1) {
                console.log(`Current Modulus: ${currentModulus}`);
                const { publicKey, privateKey } = pair(currentModulus);
                publicKeys.push(publicKey);
                privateKeys.push(privateKey);
                currentModulus = ((currentModulus / 8) + 11) * 8;
        };

        // Returns the public and private key arrays.
        return {
                privateKeys: privateKeys.reverse(),
                publicKeys
        };
};
