import { randomUUID } from 'crypto';

export function Storage() {
        const expiration = {};
        const sessions = {};
        return {
                createSession: function (variables = {}, milliseconds) {
                        // Creates a random session ID using version 4 UUID.
                        let ssid = randomUUID();
                
                        // If that session ID is not unique, keep trying until it is unique.
                        while (!(typeof sessions[ssid] !== undefined)) {
                                ssid = randomUUID();
                        };
                
                        // Injects the session into the sessions object.
                        sessions[ssid] = variables;

                        // Handles the expiration of those sessions.
                        if (
                                typeof milliseconds === 'number' &&
                                milliseconds >= 1
                        ) {
                                // Sets the expiration of the session.
                                expiration[ssid] = Date.now() + milliseconds;
                        } else {
                                // Sets the expiration of the session to infinity.
                                expiration[ssid] = Infinity;
                        };
                
                        // Returns the session ID of the session.
                        return ssid;
                },
                deleteSession: function (ssid) {
                        delete sessions[ssid];
                        delete expiration[ssid];
                },
                get: function (ssid, id) {
                        return sessions[ssid][id] || undefined;
                },
                isAuthorized: function (ssid) {
                        if (typeof sessions[ssid] !== 'object') return false;
                        if (Date.now() >= expiration[ssid]) {
                                return false;
                        };

                        return true;
                },
                set: function (ssid, id, value) {
                        sessions[ssid][id] = value;
                }
        };
};

export default { Storage };
