import * as net from 'net';

// Function to create a Networker instance
export function createServer(port, address, callback) {
        // Array to store channel handlers
        const channels = [];
        return net.createServer(function (socket) {
                // Buffer to store incoming data
                let bufferedData = Buffer.allocUnsafe(0);

                // Add a listener for 'data' event on the socket
                socket.on('data', function (newData) {
                        // Concatenate the new data to the buffered data
                        bufferedData = Buffer.concat([bufferedData, newData]);

                        // Process the buffered data as long as it is long enough to contain a complete message
                        while (bufferedData.length >= 4) {
                                const channel = bufferedData.readUint16BE();
                                const length = bufferedData.readUint16BE(2);
                                if (bufferedData.length < length + 4) break;
                                const data = bufferedData.subarray(4, 4 + length);
                                bufferedData = bufferedData.subarray(4 + length);
                                if (channels[channel]) channels[channel](data, socket);
                        };
                });
        }).listen(port, address, function () {
                // Return an object with methods to register and invoke channel handlers, and write messages to the socket
                callback({
                        on: function (channel, handler) {
                                channels[channel] = handler;
                        },
                        write: function (socket, channel, message) {
                                const data = Buffer.allocUnsafe(4 + message.length);
                                data.writeUint16BE(channel);
                                data.writeUInt16BE(message, 2);
                                message.copy(data, 2);
                                socket.write(data);
                        },
                });
        });
};

// Function to create a Networker instance
export function createClient(port, address, callback) {
        // Buffer to store incoming data
        let bufferedData = Buffer.allocUnsafe(0);

        // Array to store channel handlers
        const channels = [];

        const socket = net.createConnection(port, address, function () {
                // Return an object with methods to register and invoke channel handlers, and write messages to the socket
                callback({
                        on: function (channel, handler) {
                                channels[channel] = handler;
                        },
                        write: function (channel, message) {
                                const data = Buffer.allocUnsafe(4 + message.length);
                                data.writeUInt16BE(channel);
                                data.writeUInt16BE(message, 2);
                                message.copy(data, 4);
                                socket.write(data);
                        },
                });
        }).on('data', function (newData) {
                // Concatenate the new data to the buffered data
                bufferedData = Buffer.concat([bufferedData, newData]);

                // Process the buffered data as long as it is long enough to contain a complete message
                while (bufferedData.length >= 4) {
                        const channel = bufferedData.readUInt16BE();
                        const length = bufferedData.readUInt16BE(2);
                        if (bufferedData.length < length + 4) break;
                        const data = bufferedData.subarray(4, 4 + length);
                        bufferedData = bufferedData.subarray(4 + length);
                        if (channels[channel]) channels[channel](data);
                };
        });

        return socket;
};
