import http from 'node:http';
import https from 'node:https';

// Creates a dynamic HTTP/1.1 server. options = new https.Agent().options
export function createInsecureDynamicServer() {
        const stack = {};

        // Initializes the secure HTTP/1.1 server object. ${(options?.cert &&options?.key) ? 's' : ''}
        const server = http.createServer();

        // Request handling.
        server.on('request', function (request, response) {
                const method = request.method.toLowerCase();
                const subdomain = request.headers.host.split('.').slice(0, -2).join('.').toLowerCase();
                const { pathname } = new URL(request.url, `http://${request.headers.host}`);
                stack[subdomain]?.[method]?.[pathname]?.(request, response);
                console.log(`${request.socket.remoteAddress}:${request.socket.remotePort} ${request.method.toLowerCase()} http://${request.headers.host}:${request.socket.localPort}${request.url}`);
        });

        // Returns the object generated by the function.
        return {
                on: function (
                        path = '/',
                        {
                                allowedMethods = ['get'],
                                allowedSubdomains = ['', 'www']
                        },
                        callback = function (
                                request = new http.IncomingMessage(),
                                response = new http.ServerResponse()
                        ) {}
                ) {
                        // Ensures that all subdomains are a part of the stack.
                        for (const subdomain of allowedSubdomains) {
                                if (typeof stack[subdomain.toLowerCase()] !== 'object') {
                                        stack[subdomain.toLowerCase()] = {};
                                };

                                // Ensures that all methods are a part of the subdomain's stack.
                                for (const method of allowedMethods) {
                                        if (typeof stack[subdomain.toLowerCase()][method.toLowerCase()] !== 'object') {
                                                stack[subdomain.toLowerCase()][method.toLowerCase()] = {};
                                        };

                                        // Registers the route to the selected subdomain under the specified method.
                                        stack[subdomain.toLowerCase()][method.toLowerCase()][path.toLowerCase()] = callback;
                                        console.log(`${method.toLowerCase()}\t${subdomain.toLowerCase()}\t${path.toLowerCase()}`);
                                };
                        };
                },
                listen: function (
                        port = 80,
                        address = '0.0.0.0',
                        callback = function () {}
                ) {
                        server.listen(port, address, callback);
                },
                server
        };
};

// Creates a secure dynamic HTTP/1.1 server. options = new https.Agent().options
export function createSecureDynamicServer(options = new https.Agent().options) {
        const stack = {};

        // Initializes the secure HTTP/1.1 server object. ${(options?.cert &&options?.key) ? 's' : ''}
        const server = https.createServer(options);

        // Request handling.
        server.on('request', function (request, response) {
                const method = request.method.toLowerCase();
                const subdomain = request.headers.host.split('.').slice(0, -2).join('.').toLowerCase();
                const { pathname } = new URL(request.url, `https://${request.headers.host}`);
                stack[subdomain]?.[method]?.[pathname]?.(request, response);
                console.log(`${request.socket.remoteAddress}:${request.socket.remotePort} ${request.method.toLowerCase()} ${request.socket.localAddress}:${request.socket.localPort} https://${request.headers.host}${request.url}`);
        });

        // Returns the object generated by the function.
        return {
                on: function (
                        path = '/',
                        {
                                allowedMethods = ['get'],
                                allowedSubdomains = ['', 'www']
                        },
                        callback = function (
                                request = new http.IncomingMessage(),
                                response = new http.ServerResponse()
                        ) {}
                ) {
                        // Ensures that all subdomains are a part of the stack.
                        for (const subdomain of allowedSubdomains) {
                                if (typeof stack[subdomain.toLowerCase()] !== 'object') {
                                        stack[subdomain.toLowerCase()] = {};
                                };

                                // Ensures that all methods are a part of the subdomain's stack.
                                for (const method of allowedMethods) {
                                        if (typeof stack[subdomain.toLowerCase()][method.toLowerCase()] !== 'object') {
                                                stack[subdomain.toLowerCase()][method.toLowerCase()] = {};
                                        };

                                        // Registers the route to the selected subdomain under the specified method.
                                        stack[subdomain.toLowerCase()][method.toLowerCase()][path.toLowerCase()] = callback;
                                        console.log(`${method.toLowerCase()}\t${subdomain.toLowerCase()}\t${path.toLowerCase()}`);
                                };
                        };
                },
                listen: function (
                        port = 443,
                        address = '0.0.0.0',
                        callback = function () {}
                ) {
                        server.listen(port, address, callback);
                },
                server
        };
};

// Exports everything under a default variable. (options?.cert &&options?.key) ? 443 : 
export default {
        createInsecureDynamicServer,
        createSecureDynamicServer
};
