import http2 from 'node:http2';

// Creates an HTTP/2 server.
export function createServer(options) {
        const stack = {};

        // Initializes the secure HTTP/1.1 server object.
        const server = (
                options?.cert &&
                options?.key
        ) ?
                http2.createSecureServer(options) :
                http2.createServer();

        // Request handling.
        server.on('stream', function (stream, headers) {
                const method = headers?.[':method']?.toLowerCase();
                const subdomain = headers?.[':authority']?.split('.').slice(0, -2).join('.').toLowerCase();
                const { pathname } = new URL(headers?.[':path'], `http${(
                        options?.cert &&
                        options?.key
                ) ? 's' : ''}://${headers?.[':authority']}`);
                stack[subdomain]?.[method]?.[pathname]?.(stream, headers);
                console.log(method, subdomain, pathname, headers?.[':authority']);
        });

        // Returns the object generated by the function.
        return {
                on: function (
                        path = '/',
                        {
                                allowedMethods = ['get'],
                                allowedSubdomains = ['', 'www']
                        },
                        callback = function (
                                stream,
                                headers
                        ) {}
                ) {
                        // Ensures that all subdomains are a part of the stack.
                        for (const subdomain of allowedSubdomains) {
                                if (typeof stack[subdomain.toLowerCase()] !== 'object') {
                                        stack[subdomain.toLowerCase()] = {};
                                };

                                // Ensures that all methods are a part of the subdomain's stack.
                                for (const method of allowedMethods) {
                                        if (typeof stack[subdomain.toLowerCase()][method.toLowerCase()] !== 'object') {
                                                stack[subdomain.toLowerCase()][method.toLowerCase()] = {};
                                        };

                                        // Registers the route to the selected subdomain under the specified method.
                                        stack[subdomain.toLowerCase()][method.toLowerCase()][path.toLowerCase()] = callback;
                                };
                        };
                },
                listen: function (
                        port = (
                                options?.cert &&
                                options?.key
                        ) ? 443 : 80,
                        address = '0.0.0.0',
                        callback = function () {}
                ) {
                        server.listen(port, address, callback);
                },
                server
        };
};

// Exports everything under a default variable.
export default {
        createServer
};
